/**
 ******************************************************************************
 * @file    I2CDriver.h
 * @author  STMicroelectronics - ST-Korea - MCD Team
 * @version 1.2.0
 * @date    Sep 7, 2016
 *
 * @brief   I2C driver declaration.
 *
 * The I2C driver implements the generic Driver interface.
 *
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT 2016 STMicroelectronics</center></h2>
 *
 * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *        http://www.st.com/software_license_agreement_liberty_v2
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ******************************************************************************
 */

#ifndef INCLUDE_DRIVERS_I2CDRIVER_H_

#define INCLUDE_DRIVERS_I2CDRIVER_H_

#ifdef __cplusplus
extern "C" {
#endif

//#include "stm32l0xx_hal.h"
#include "IDriver.h"
#include "IDriverVtbl.h"
#include "II2CDriverListener.h"

/**
 * Creates a type name for _I2CDriver.
 */
typedef struct _I2CDriver I2CDriver;

/**
 * I2C Driver class implements the IDriver interface.
 */
struct _I2CDriver {
	/**
	 * Base class object.
	 */
	IDriver super;

	/**
	 * Handle to the low level peripheral;
	 */
	I2C_HandleTypeDef m_xHi2c;

	/**
	 * Specifies an index in the s_pxHwResouces array. It is used by the driver to access
	 * the hardware resources. It depends on the I2C peripheral passed to the driver Init method.
	 */
	uint8_t m_nHRIndex;

	/**
	 * specifies the timeout for the read and write operation in tick. The default value is portMAX_DELAY (wait forever);
	 */
	uint32_t  m_nTimeout;
};

/**
 * Allocate an instance of I2CDriver. The driver is allocated
 * in the FreeRTOS heap.
 *
 * @return a pointer to the generic interface ::IDriver if success,
 * or SYS_OUT_OF_MEMORY_ERROR_CODE otherwise.
 */
IDriver *I2CDrvAlloc();

// I2C Specific API

/**
 * Change the configuration used by the driver to read from the bus. This function is related with
 * the underlying STM32 HAL library. Fro more information see also the STM32L0 HAL documentation
 * available at: http://www.st.com/resource/en/user_manual/dm00113898.pdf
 *
 * @param this [IN] specifies a pointer to an I2CDriver object.
 * @param nMode [IN] can be one of:
 *                   - I2C_RELOAD_MODE
 *                   - I2C_AUTOEND_MODE
 *                   - I2C_SOFTEND_MODE
 * @param nRequest [IN] can be one of:
 *                      - I2C_NO_STARTSTOP
 *                      - I2C_GENERATE_STOP
 *                      - I2C_GENERATE_START_READ
 *                      - I2C_GENERATE_START_WRITE
 * @return SYS_NO_ERROR_CODE
 */
sys_error_code_t I2CDrvUpdateReadConfiguration(IDriver *this, uint32_t nMode, uint32_t nRequest);

/**
 * Set the timeout for the read and write operation.
 *
 * @param this [IN] specifies a pointer to an I2CDriver object.
 * @param nTimeoutMS [IN] specifies the new timeout in ms.
 * @return SYS_NO_ERROR_CODE
 */
sys_error_code_t I2CDrvSetTimeout(IDriver *this, uint32_t nTimeoutMS);

/**
 *  Connect or disconnect the PINs to the peripheral. When the PINs are disconnected they are configured in analog no pull.
 *
 * @param this [IN] specifies a driver object.
 * @param bConnect [IN] TRUE to connect the PINs, FALSE to disconnect the PINs.
 * @return SYS_NO_ERROR_CODE if success, an error code otherwise.
 */
sys_error_code_t I2CDrvConnectPINs(IDriver *this, boolean_t bConnect);

/**
 * Read `nDataSize` byte in Non Blocking node. The calling task is not blocked and the function return
 * before the data are read for the peripheral. When the operation is completed, then the driver
 * notify the listener from the ISR.
 *
 * @param this [IN] specifies a driver object.
 * @param pDataBuffer
 * @param nDataSize
 * @param nChannel
 * @param pListener
 * @return
 */
sys_error_code_t I2CDrvNBRead(IDriver *this, uint8_t *pDataBuffer, uint16_t nDataSize, uint16_t nChannel, IListener *pListener);


// Inline functions definition
// ***************************


#ifdef __cplusplus
}
#endif


#endif /* INCLUDE_DRIVERS_I2CDRIVER_H_ */
