/**
 ******************************************************************************
 * @file    I2CSlaveDriver.c
 * @author  STMicroelectronics - ST-Korea - MCD Team
 * @version 1.0.0
 * @date    Oct 24, 2017
 *
 * @brief
 *
 * <DESCRIPTIOM>
 *
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT 2017 STMicroelectronics</center></h2>
 *
 * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *        http://www.st.com/software_license_agreement_liberty_v2
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ******************************************************************************
 */

#include "I2CDriver.h"
#include "I2CDriverVtbl.h"
#include "II2CDriverListener_vtbl.h"
#include "FreeRTOS.h"
#include "semphr.h"
#include "sysdebug.h"

#define SYS_DEBUGF(level, message) 			SYS_DEBUGF3(SYS_DBG_DRIVERS, level, message)

#define I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT  1

// I2C1

#define I2CDRV_I2C1_HR_IDX                     0

#ifndef I2CDRV_CFG_1_IRQ_LINE
#define I2CDRV_CFG_1_IRQ_LINE                  I2C2_EV_IRQn
#endif

#ifndef I2CDRV_CFG_1_IRQ_PRIORITY
#define I2CDRV_CFG_1_IRQ_PRIORITY              8
#endif

#ifndef I2CDRV_CFG_1_IRQ_ERR_LINE
#define I2CDRV_CFG_1_IRQ_ERR_LINE              I2C2_ER_IRQn
#endif


#ifndef I2CDRV_CFG_1_IRQ_ERR_PRIORITY
#define I2CDRV_CFG_1_IRQ_ERR_PRIORITY          I2CDRV_CFG_1_IRQ_PRIORITY
#endif

#define I2CDRV_CFG_1_SLAVE_DDRESS              0x54

// I2C3

#define I2CDRV_I2C3_HR_IDX                     1

#ifndef I2CDRV_CFG_3_IRQ_LINE
#define I2CDRV_CFG_3_IRQ_LINE                  I2C3_IRQn
#endif

#ifndef I2CDRV_CFG_3_IRQ_PRIORITY
#define I2CDRV_CFG_3_IRQ_PRIORITY              9
#endif


/**
 * I2C Driver virtual table.
 */
static const IDriver_vtbl s_xI2CDrv_vtbl = {
    I2CDrvInit,
    I2CDrvStart,
    I2CDrvStop,
    I2CDrvDoEnterPowerMode,
    I2CDrvWrite,
    I2CDrvRead
};

/**
 * Data associated to the hardware peripheral.
 */
typedef struct _I2CPeripheralResources {
  /**
   * HAL handle used by the I2Cx IRQ handler.
   */
  I2C_HandleTypeDef *m_pxI2CHandle;

  /**
   * Synchronization object used by the driver to synchronize the I2C ISR with the task using the driver;
   */
  SemaphoreHandle_t m_xSyncObj;

  /**
   * Specifies the listener object to be notified when the non blocking (NB) read is used
   */
  IListener *m_pxListener;
}I2CPeripheralResources;

static I2CPeripheralResources s_pxHwResouces[I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT] = {
  {NULL, NULL, NULL}
};


#if (SYS_DBG_ENABLE_TA4 >= 1)
static traceString s_xSTF12Msg;
#endif
#if (SYS_DBG_ENABLE_TA4 == 2)
static traceHandle s_xI2C1EVHandle;
static traceHandle s_xI2C1ERHandle;
#endif

// Inline function forward declaration.
#if defined (__GNUC__)
extern sys_error_code_t II2CDriverListenerOnReadNB(IListener *this);
extern void II2CDriverListenerSetOwner(IListener *this, void *pOwner);
extern void *II2CDriverListenerGetOwner(IListener *this);
#endif

// Private member function declaration
// ***********************************


// Public API definition
// *********************

IDriver *I2CDrvAlloc() {
  IDriver *pNewObj = (IDriver*)pvPortMalloc(sizeof(I2CDriver));

  if (pNewObj == NULL) {
    SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_OUT_OF_MEMORY_ERROR_CODE);
    SYS_DEBUGF(SYS_DBG_LEVEL_WARNING, ("I2CDrv - alloc failed.\r\n"));
  }

  else {
    pNewObj->vptr = &s_xI2CDrv_vtbl;
  }

  return pNewObj;
}

sys_error_code_t I2CDrvInit(IDriver *this, void *pParams) {
  assert_param(this);
  assert_param(pParams);

  I2CDriver *pObj = (I2CDriver*)this;

  // The following code is copied from the function MX_I2C1_Init generated by STM32CubeMX
  pObj->m_xHi2c.Instance = (I2C_TypeDef *)pParams;
  pObj->m_xHi2c.Init.Timing = 0x00500615;
  pObj->m_xHi2c.Init.OwnAddress1 = I2CDRV_CFG_1_SLAVE_DDRESS;
  pObj->m_xHi2c.Init.AddressingMode = I2C_ADDRESSINGMODE_7BIT;
  pObj->m_xHi2c.Init.DualAddressMode = I2C_DUALADDRESS_DISABLE;
  pObj->m_xHi2c.Init.OwnAddress2 = 0x0;
  pObj->m_xHi2c.Init.OwnAddress2Masks = I2C_OA2_NOMASK;
  pObj->m_xHi2c.Init.GeneralCallMode = I2C_GENERALCALL_DISABLE;
  pObj->m_xHi2c.Init.NoStretchMode = I2C_NOSTRETCH_DISABLE;
  if (HAL_I2C_Init(&pObj->m_xHi2c) != HAL_OK)
  {
    SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_I2C_INIT_ERROR_CODE);
    SYS_DEBUGF(SYS_DBG_LEVEL_WARNING, ("I2CDrv: error during I2C initialization.\r\n"));
    return SYS_I2C_INIT_ERROR_CODE;
  }

    //Configure Analog filter
  if (HAL_I2CEx_ConfigAnalogFilter(&pObj->m_xHi2c, I2C_ANALOGFILTER_ENABLE) != HAL_OK)
  {
    SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_I2C_INIT_ERROR_CODE);
    SYS_DEBUGF(SYS_DBG_LEVEL_WARNING, ("I2CDrv: error during I2C initialization.\r\n"));
    return SYS_I2C_INIT_ERROR_CODE;
  }

  //Configure Digital filter
  if (HAL_I2CEx_ConfigDigitalFilter(&pObj->m_xHi2c, 0) != HAL_OK)
  {
    SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_I2C_INIT_ERROR_CODE);
    SYS_DEBUGF(SYS_DBG_LEVEL_WARNING, ("I2CDrv: error during I2C initialization.\r\n"));
    return SYS_I2C_INIT_ERROR_CODE;
  }

  // Enable the wakeup from STOP mode.
  HAL_I2CEx_EnableWakeUp(&pObj->m_xHi2c);

  // Initialize the software resources for the I2Cx peripheral.
  pObj->m_nTimeout = portMAX_DELAY;
  if (pObj->m_xHi2c.Instance == I2C2) {
    pObj->m_nHRIndex = I2CDRV_I2C1_HR_IDX;
  }
#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (pObj->m_xHi2c.Instance == I2C3) {
    pObj->m_nHRIndex = I2CDRV_I2C3_HR_IDX;
  }
#endif
  else {
    // this is for a safety check. If the the initialization parameter is not
    // one of the two I2C peripheral supported the hardware resource is not initialized.
    pObj->m_nHRIndex = 0xFF;
  }

  if ((pObj->m_nHRIndex != 0xFF) && s_pxHwResouces[pObj->m_nHRIndex].m_pxI2CHandle == NULL) {
    s_pxHwResouces[pObj->m_nHRIndex].m_pxI2CHandle = &pObj->m_xHi2c;
    s_pxHwResouces[pObj->m_nHRIndex].m_xSyncObj = xSemaphoreCreateBinary();
    if (s_pxHwResouces[pObj->m_nHRIndex].m_xSyncObj == NULL) {
      SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_I2C_INIT_ERROR_CODE);
      SYS_DEBUGF(SYS_DBG_LEVEL_WARNING, ("I2CDrv: unable to create sync obj.\r\n"));
      return SYS_I2C_INIT_ERROR_CODE;
    }

#ifdef DEBUG
    vQueueAddToRegistry(s_pxHwResouces[pObj->m_nHRIndex].m_xSyncObj, "I2CDrv");
#endif
  }

#if (SYS_DBG_ENABLE_TA4 >= 1)
  s_xSTF12Msg = xTraceRegisterString("STF12-I2C");
#endif
#if (SYS_DBG_ENABLE_TA4 == 2)
  s_xI2C1EVHandle = xTraceSetISRProperties("ISRI2C2_EV", I2CDRV_CFG_1_IRQ_PRIORITY);
  s_xI2C1ERHandle = xTraceSetISRProperties("ISRI2C2_ER", I2CDRV_CFG_1_IRQ_PRIORITY);
#endif

  SYS_DEBUGF(SYS_DBG_LEVEL_VERBOSE, ("I2CDrv: initialization done.\r\n"));

  return SYS_NO_ERROR_CODE;
}

sys_error_code_t I2CDrvStart(IDriver *this) {
  assert_param(this);
  I2CDriver *pObj = (I2CDriver*)this;

  if (pObj->m_nHRIndex == I2CDRV_I2C1_HR_IDX) {
    HAL_NVIC_EnableIRQ(I2CDRV_CFG_1_IRQ_LINE);
    HAL_NVIC_EnableIRQ(I2CDRV_CFG_1_IRQ_ERR_LINE);
  }
#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (pObj->m_nHRIndex == I2CDRV_I2C3_HR_IDX) {
    HAL_NVIC_EnableIRQ(I2CDRV_CFG_3_IRQ_LINE);
  }
#endif

  return SYS_NO_ERROR_CODE;
}

sys_error_code_t I2CDrvStop(IDriver *this) {
  assert_param(this);
  I2CDriver *pObj = (I2CDriver*)this;

  if (pObj->m_nHRIndex == I2CDRV_I2C1_HR_IDX) {
    HAL_NVIC_DisableIRQ(I2CDRV_CFG_1_IRQ_LINE);
    HAL_NVIC_ClearPendingIRQ(I2CDRV_CFG_1_IRQ_LINE);
    HAL_NVIC_DisableIRQ(I2CDRV_CFG_1_IRQ_ERR_LINE);
    HAL_NVIC_ClearPendingIRQ(I2CDRV_CFG_1_IRQ_ERR_LINE);

  }
#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (pObj->m_nHRIndex == I2CDRV_I2C3_HR_IDX) {
    HAL_NVIC_DisableIRQ(I2CDRV_CFG_3_IRQ_LINE);
    HAL_NVIC_ClearPendingIRQ(I2CDRV_CFG_3_IRQ_LINE);
  }
#endif

  return SYS_NO_ERROR_CODE;
}

sys_error_code_t I2CDrvDoEnterPowerMode(IDriver *this, const EPowerMode eActivePowerMode, const EPowerMode eNewPowerMode) {
  assert_param(this);
  sys_error_code_t xRes = SYS_NO_ERROR_CODE;
//	I2CDriver *pObj = (I2CDriver*)this;
  UNUSED(this);

  return xRes;
}

sys_error_code_t I2CDrvWrite(IDriver *this, uint8_t *pDataBuffer, uint16_t nDataSize, uint16_t nChannel) {
  assert_param(this);
  sys_error_code_t xRes = SYS_NO_ERROR_CODE;
  I2CDriver *pObj = (I2CDriver*)this;

  if (s_pxHwResouces[pObj->m_nHRIndex].m_pxListener != NULL) {
    xRes = SYS_I2C_IO_PENDING_ERROR_CODE;
  }
  else {
    while (HAL_I2C_GetState(&pObj->m_xHi2c) != HAL_I2C_STATE_READY) {
      __asm volatile( "NOP" );
    }
    if (HAL_I2C_Slave_Transmit_IT(&pObj->m_xHi2c, pDataBuffer, nDataSize)!= HAL_OK) {
      sys_error_handler();
    }

    // Suspend the calling task until the operation is completed.
    if (pdTRUE != xSemaphoreTake(s_pxHwResouces[pObj->m_nHRIndex].m_xSyncObj, pObj->m_nTimeout)) {
      SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_I2C_TIMEOUT_ERROR_CODE);
      xRes = SYS_I2C_TIMEOUT_ERROR_CODE;
    }
  }

  return xRes;
}

sys_error_code_t I2CDrvRead(IDriver *this, uint8_t *pDataBuffer, uint16_t nDataSize, uint16_t nChannel) {
  assert_param(this);
  sys_error_code_t xRes = SYS_NO_ERROR_CODE;
  I2CDriver *pObj = (I2CDriver*)this;

  if (s_pxHwResouces[pObj->m_nHRIndex].m_pxListener != NULL) {
    xRes = SYS_I2C_IO_PENDING_ERROR_CODE;
  }
  else {
    while (HAL_I2C_GetState(&pObj->m_xHi2c) != HAL_I2C_STATE_READY) {
      __asm volatile( "NOP" );
    }
    if (HAL_I2C_Slave_Receive_IT(&pObj->m_xHi2c, pDataBuffer, nDataSize)!= HAL_OK) {
      sys_error_handler();
    }

    // Suspend the calling task until the operation is completed.
    if (pdTRUE != xSemaphoreTake(s_pxHwResouces[pObj->m_nHRIndex].m_xSyncObj, pObj->m_nTimeout)) {
      SYS_SET_LOW_LEVEL_ERROR_CODE(SYS_I2C_TIMEOUT_ERROR_CODE);
      xRes = SYS_I2C_TIMEOUT_ERROR_CODE;
    }
  }

  return xRes;
}

sys_error_code_t I2CDrvNBRead(IDriver *this, uint8_t *pDataBuffer, uint16_t nDataSize, uint16_t nChannel, IListener *pListener) {
  assert_param(this);
  sys_error_code_t xRes = SYS_NO_ERROR_CODE;
  I2CDriver *pObj = (I2CDriver*)this;

  if (s_pxHwResouces[pObj->m_nHRIndex].m_pxListener != NULL) {
    xRes = SYS_I2C_IO_PENDING_ERROR_CODE;
  }
  else {
    while (HAL_I2C_GetState(&pObj->m_xHi2c) != HAL_I2C_STATE_READY) {
      __asm volatile( "NOP" );
    }

    // set the listener
    s_pxHwResouces[pObj->m_nHRIndex].m_pxListener = pListener;

    if (HAL_I2C_Slave_Receive_IT(&pObj->m_xHi2c, pDataBuffer, nDataSize)!= HAL_OK) {
      sys_error_handler();
    }
  }

  return xRes;
}

sys_error_code_t I2CDrvUpdateReadConfiguration(IDriver *this, uint32_t nMode, uint32_t nRequest) {
  assert_param(this);
  assert_param((nMode == I2C_RELOAD_MODE) || (nMode == I2C_AUTOEND_MODE) || (nMode == I2C_SOFTEND_MODE));
  assert_param((nRequest == I2C_NO_STARTSTOP) || (nRequest == I2C_GENERATE_STOP) || (nRequest == I2C_GENERATE_START_READ) || (nRequest == I2C_GENERATE_START_WRITE));

#ifdef USE_MODIFIED_HAL_DRIVER
  I2CDriver *pObj = (I2CDriver*)this;
  pObj->m_xHi2c.Init.ReadMode = nMode;
  pObj->m_xHi2c.Init.ReadRequest = nRequest;
#endif

  return SYS_NO_ERROR_CODE;
}

sys_error_code_t I2CDrvSetTimeout(IDriver *this, uint32_t nTimeoutMS) {
  assert_param(this);
  I2CDriver *pObj = (I2CDriver*)this;

  pObj->m_nTimeout = (nTimeoutMS != portMAX_DELAY) ? pdMS_TO_TICKS(nTimeoutMS) : nTimeoutMS;

  return SYS_NO_ERROR_CODE;
}

sys_error_code_t I2CDrvConnectPINs(IDriver *this, boolean_t bConnect) {
  assert_param(this);
  I2CDriver *pObj = (I2CDriver*)this;
  GPIO_InitTypeDef GPIO_InitStruct;

  if (pObj->m_xHi2c.Instance == I2C2) {
    if (bConnect) {
      // I2C1 GPIO Configuration
      // PB10    ------> I2C1_SCL
      // PB11    ------> I2C1_SDA
      GPIO_InitStruct.Pin = GPIO_PIN_10|GPIO_PIN_11;
      GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
      GPIO_InitStruct.Pull = GPIO_PULLUP;
      GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
      GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
      HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
    }
    else {
      // configure the I2C PINs in Analog no pull.
      // PB10     ------> I2C1_SCL
      // PB11     ------> I2C1_SDA
      GPIO_InitStruct.Pin = GPIO_PIN_10|GPIO_PIN_11;
      GPIO_InitStruct.Mode = GPIO_MODE_ANALOG;
      GPIO_InitStruct.Pull = GPIO_NOPULL;
      HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);
    }
  }

  return SYS_NO_ERROR_CODE;
}

// Private API definition
// **********************


// CubeMX integration functions
// ****************************

void HAL_I2C_MspInit(I2C_HandleTypeDef* hi2c)
{
  GPIO_InitTypeDef GPIO_InitStruct;

  if (hi2c->Instance == I2C2) {
    // Enable the GPIO clock for SCL and SDA PIN
    __HAL_RCC_GPIOB_CLK_ENABLE();

    // I2C1 GPIO Configuration
    // PB10    ------> I2C1_SCL
    // PB11    ------> I2C1_SDA
    GPIO_InitStruct.Pin = GPIO_PIN_10|GPIO_PIN_11;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
    GPIO_InitStruct.Pull = GPIO_NOPULL;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF4_I2C2;
    HAL_GPIO_Init(GPIOB, &GPIO_InitStruct);

    // Peripheral clock enable
    __HAL_RCC_I2C2_CLK_ENABLE();
    // Peripheral interrupt init
    HAL_NVIC_SetPriority(I2CDRV_CFG_1_IRQ_LINE, I2CDRV_CFG_1_IRQ_PRIORITY, 0);
    HAL_NVIC_SetPriority(I2CDRV_CFG_1_IRQ_ERR_LINE, I2CDRV_CFG_1_IRQ_ERR_PRIORITY, 0);
  }

#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (hi2c->Instance == I2C3) {
    __HAL_RCC_GPIOC_CLK_ENABLE();

    // I2C3 GPIO Configuration
    // PC1     ------> I2C3_SDA
    // PC0     ------> I2C3_SCL
    GPIO_InitStruct.Pin = GPIO_PIN_1|GPIO_PIN_0;
    GPIO_InitStruct.Mode = GPIO_MODE_AF_OD;
    GPIO_InitStruct.Pull = GPIO_PULLUP;
    GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_VERY_HIGH;
    GPIO_InitStruct.Alternate = GPIO_AF7_I2C3;
    HAL_GPIO_Init(GPIOC, &GPIO_InitStruct);

    // Peripheral clock enable
    __HAL_RCC_I2C3_CLK_ENABLE();

    // Peripheral interrupt init
    HAL_NVIC_SetPriority(I2CDRV_CFG_3_IRQ_LINE, I2CDRV_CFG_3_IRQ_PRIORITY, 0);
  }
#endif
}

void HAL_I2C_MspDeInit(I2C_HandleTypeDef* hi2c)
{
  if (hi2c->Instance == I2C2) {
    // Reset peripheral
    __HAL_RCC_I2C2_FORCE_RESET();
    __HAL_RCC_I2C2_RELEASE_RESET();
    // Peripheral clock disable
    __HAL_RCC_I2C2_CLK_DISABLE();


    // I2C1 GPIO Configuration
    // PB10    ------> I2C1_SCL
    // PB11    ------> I2C1_SDA
    HAL_GPIO_DeInit(GPIOB, GPIO_PIN_10|GPIO_PIN_11);

    // Peripheral interrupt DeInit
    HAL_NVIC_DisableIRQ(I2CDRV_CFG_1_IRQ_LINE);
    HAL_NVIC_ClearPendingIRQ(I2CDRV_CFG_1_IRQ_LINE);
    HAL_NVIC_DisableIRQ(I2CDRV_CFG_1_IRQ_ERR_LINE);
    HAL_NVIC_ClearPendingIRQ(I2CDRV_CFG_1_IRQ_ERR_LINE);
  }

#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (hi2c->Instance == I2C3) {
    // Reset peripheral
    __HAL_RCC_I2C3_FORCE_RESET();
    __HAL_RCC_I2C3_RELEASE_RESET();
    // Peripheral clock disable
   __HAL_RCC_I2C3_CLK_DISABLE();

    // I2C3 GPIO Configuration
    // PC1     ------> I2C3_SDA
    // PC0     ------> I2C3_SCL
    HAL_GPIO_DeInit(GPIOC, GPIO_PIN_0|GPIO_PIN_1);

    // Peripheral interrupt Deinit
    HAL_NVIC_DisableIRQ(I2CDRV_CFG_3_IRQ_LINE);
    HAL_NVIC_ClearPendingIRQ(I2CDRV_CFG_3_IRQ_LINE);
  }
#endif
}

/**
  * I2C error callback.
  *
  * @param  hi2c : Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  */
void HAL_I2C_ErrorCallback(I2C_HandleTypeDef *hi2c)
{
  // Prevent unused argument(s) compilation warning
  UNUSED(hi2c);

  switch (hi2c->ErrorCode) {
  default:
    break;
  }

#if (SYS_DBG_ENABLE_TA4 >= 1)
//  vTraceStoreISREnd(0);
  vTracePrintF(s_xSTF12Msg, "I2C ERR:%i", hi2c->ErrorCode);
#endif

  sys_error_handler();
}

/**
  *  Slave Tx Transfer completed callback.
  *
  * @param  hi2c : Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  */
void HAL_I2C_SlaveTxCpltCallback(I2C_HandleTypeDef *hi2c) {
  static BaseType_t xHigherPriorityTaskWoken;
  xHigherPriorityTaskWoken = pdFALSE;
  // Notify the task
  if (hi2c->Instance == I2C2) {
    xSemaphoreGiveFromISR(s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_xSyncObj, &xHigherPriorityTaskWoken);
  }
#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (hi2c->Instance == I2C3) {
    xSemaphoreGiveFromISR(s_pxHwResouces[I2CDRV_I2C3_HR_IDX].m_xSyncObj, &xHigherPriorityTaskWoken);
  }
#endif
  portEND_SWITCHING_ISR(xHigherPriorityTaskWoken);
}

/**
  * Slave Rx Transfer completed callback.
  *
  * @param  hi2c : Pointer to a I2C_HandleTypeDef structure that contains
  *                the configuration information for the specified I2C.
  */
void HAL_I2C_SlaveRxCpltCallback(I2C_HandleTypeDef *hi2c) {
  static BaseType_t xHigherPriorityTaskWoken;
  xHigherPriorityTaskWoken = pdFALSE;
  // Notify the task
  if (hi2c->Instance == I2C2) {
    // Non Blocking request
    if (s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_pxListener != NULL) {
      II2CDriverListenerOnReadNB(s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_pxListener);
      s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_pxListener = NULL;
    }
    else {
      // Blocking Request
      xSemaphoreGiveFromISR(s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_xSyncObj, &xHigherPriorityTaskWoken);
    }
  }
#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
  else if (hi2c->Instance == I2C3) {
    xSemaphoreGiveFromISR(s_pxHwResouces[I2CDRV_I2C3_HR_IDX].m_xSyncObj, &xHigherPriorityTaskWoken);
  }
#endif
  portEND_SWITCHING_ISR(xHigherPriorityTaskWoken);
}

/**
* This function handles I2C2 event interrupt.
*/
void I2C2_EV_IRQHandler(void)
{
#if (SYS_DBG_ENABLE_TA4 == 2)
  vTraceStoreISRBegin(s_xI2C1EVHandle);
#endif

  I2C_HandleTypeDef *pxI2C = s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_pxI2CHandle;

  HAL_I2C_EV_IRQHandler(pxI2C);

#if (SYS_DBG_ENABLE_TA4 == 2)
  vTraceStoreISREnd(0);
#endif
}

/**
* This function handles I2C2 error interrupt.
*/
void I2C2_ER_IRQHandler(void)
{
#if (SYS_DBG_ENABLE_TA4 == 2)
  vTraceStoreISRBegin(s_xI2C1ERHandle);
#endif

  I2C_HandleTypeDef *pxI2C = s_pxHwResouces[I2CDRV_I2C1_HR_IDX].m_pxI2CHandle;

  HAL_I2C_ER_IRQHandler(pxI2C);

#if (SYS_DBG_ENABLE_TA4 == 2)
  vTraceStoreISREnd(0);
#endif
}

#if I2CDRV_CFG_HARDWARE_PERIPHERALS_COUNT > 1
/**
  * This function handles I2C3 event and error interrupt request.
  */
void I2C3_IRQHandler(void)
{
  I2C_HandleTypeDef *pxI2C = s_pxHwResouces[I2CDRV_I2C3_HR_IDX].m_pxI2CHandle;

  if (pxI2C->Instance->ISR & (I2C_FLAG_BERR | I2C_FLAG_ARLO | I2C_FLAG_OVR)) {
    HAL_I2C_ER_IRQHandler(pxI2C);
  }
  else {
    HAL_I2C_EV_IRQHandler(pxI2C);
  }
}
#endif

